@@ -4,7 +4,7 @@
  * @author Dave Longley
  *
  * @license BSD 3-Clause License
- * Copyright (c) 2011-2017 Digital Bazaar, Inc.
+ * Copyright (c) 2011-2019 Digital Bazaar, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,8 +35,10 @@
  */
 const canonize = require('rdf-canonize');
 const util = require('./util');
+const ContextResolver = require('./ContextResolver');
 const IdentifierIssuer = util.IdentifierIssuer;
 const JsonLdError = require('./JsonLdError');
+const LRU = require('lru-cache');
 const NQuads = require('./NQuads');
 const Rdfa = require('./Rdfa');
 
@@ -46,7 +48,8 @@ const {fromRDF: _fromRDF} = require('./fromRdf');
 const {toRDF: _toRDF} = require('./toRdf');
 
 const {
-  frameMergedOrDefault: _frameMergedOrDefault
+  frameMergedOrDefault: _frameMergedOrDefault,
+  cleanupNull: _cleanupNull
 } = require('./frame');
 
 const {
@@ -60,15 +63,15 @@ const {
 } = require('./graphTypes');
 
 const {
+  expandIri: _expandIri,
   getInitialContext: _getInitialContext,
   process: _processContext,
-  getAllContexts: _getAllContexts
+  processingMode: _processingMode
 } = require('./context');
 
 const {
   compact: _compact,
-  compactIri: _compactIri,
-  removePreserve: _removePreserve
+  compactIri: _compactIri
 } = require('./compact');
 
 const {
@@ -77,7 +80,7 @@ const {
   mergeNodeMaps: _mergeNodeMaps
 } = require('./nodeMap');
 
-// determine if in-browser or using node.js
+// determine if in-browser or using Node.js
 const _nodejs = (
   typeof process !== 'undefined' && process.versions && process.versions.node);
 const _browser = !_nodejs &&
@@ -87,6 +90,14 @@ const _browser = !_nodejs &&
 // attaches jsonld API to the given object
 const wrapper = function(jsonld) {
 
+/** Registered RDF dataset parsers hashed by content-type. */
+const _rdfParsers = {};
+
+// resolved context cache
+// TODO: consider basing max on context size rather than number
+const RESOLVED_CONTEXT_CACHE_MAX_SIZE = 100;
+const _resolvedContextCache = new LRU({max: RESOLVED_CONTEXT_CACHE_MAX_SIZE});
+
 /* Core API */
 
 /**
@@ -104,7 +115,7 @@ const wrapper = function(jsonld) {
  *          [expandContext] a context to expand with.
  *          [skipExpansion] true to assume the input is expanded and skip
  *            expansion, false not to, defaults to false.
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
+ *          [documentLoader(url, options)] the document loader.
  *          [expansionMap(info)] a function that can be used to custom map
  *            unmappable values (or to throw an error when they are detected);
  *            if this function returns `undefined` then the default behavior
@@ -114,11 +125,11 @@ const wrapper = function(jsonld) {
  *            unmappable values (or to throw an error when they are detected);
  *            if this function returns `undefined` then the default behavior
  *            will be used.
- * @param [callback(err, compacted)] called once the operation completes.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the compacted output.
  */
-jsonld.compact = util.callbackify(async function(input, ctx, options) {
+jsonld.compact = async function(input, ctx, options) {
   if(arguments.length < 2) {
     throw new TypeError('Could not compact, too few arguments.');
   }
@@ -142,7 +153,9 @@ jsonld.compact = util.callbackify(async function(input, ctx, options) {
     graph: false,
     skipExpansion: false,
     link: false,
-    issuer: new IdentifierIssuer('_:b')
+    issuer: new IdentifierIssuer('_:b'),
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
   });
   if(options.link) {
     // force skip expansion when linking, "link" is not part of the public
@@ -166,7 +179,7 @@ jsonld.compact = util.callbackify(async function(input, ctx, options) {
     _getInitialContext(options), ctx, options);
 
   // do compaction
-  let compacted = _compact({
+  let compacted = await _compact({
     activeCtx,
     element: expanded,
     options,
@@ -233,18 +246,8 @@ jsonld.compact = util.callbackify(async function(input, ctx, options) {
     }
   }
 
-  if(options.framing) {
-    // get graph alias
-    const graph = _compactIri({
-      activeCtx, iri: '@graph', relativeTo: {vocab: true}
-    });
-    // remove @preserve from results
-    options.link = {};
-    compacted[graph] = _removePreserve(activeCtx, compacted[graph], options);
-  }
-
   return compacted;
-});
+};
 
 /**
  * Performs JSON-LD expansion.
@@ -255,23 +258,25 @@ jsonld.compact = util.callbackify(async function(input, ctx, options) {
  *          [expandContext] a context to expand with.
  *          [keepFreeFloatingNodes] true to keep free-floating nodes,
  *            false not to, defaults to false.
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
+ *          [documentLoader(url, options)] the document loader.
  *          [expansionMap(info)] a function that can be used to custom map
  *            unmappable values (or to throw an error when they are detected);
  *            if this function returns `undefined` then the default behavior
  *            will be used.
- * @param [callback(err, expanded)] called once the operation completes.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the expanded output.
  */
-jsonld.expand = util.callbackify(async function(input, options) {
+jsonld.expand = async function(input, options) {
   if(arguments.length < 1) {
     throw new TypeError('Could not expand, too few arguments.');
   }
 
   // set default options
   options = _setDefaults(options, {
-    keepFreeFloatingNodes: false
+    keepFreeFloatingNodes: false,
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
   });
   if(options.expansionMap === false) {
     options.expansionMap = undefined;
@@ -316,17 +321,14 @@ jsonld.expand = util.callbackify(async function(input, options) {
     options.base = defaultBase || '';
   }
 
-  // get all contexts in `toResolve`
-  await _getAllContexts(toResolve, options);
-
   // process any additional contexts
   let activeCtx = _getInitialContext(options);
-  contextsToProcess.forEach(localCtx => {
-    activeCtx = _processContext({activeCtx, localCtx, options});
-  });
+  for(const localCtx of contextsToProcess) {
+    activeCtx = await _processContext({activeCtx, localCtx, options});
+  }
 
   // expand resolved input
-  let expanded = _expand({
+  let expanded = await _expand({
     activeCtx,
     element: toResolve.input,
     options,
@@ -347,7 +349,7 @@ jsonld.expand = util.callbackify(async function(input, options) {
   }
 
   return expanded;
-});
+};
 
 /**
  * Performs JSON-LD flattening.
@@ -357,12 +359,12 @@ jsonld.expand = util.callbackify(async function(input, options) {
  * @param [options] the options to use:
  *          [base] the base IRI to use.
  *          [expandContext] a context to expand with.
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
- * @param [callback(err, flattened)] called once the operation completes.
+ *          [documentLoader(url, options)] the document loader.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the flattened output.
  */
-jsonld.flatten = util.callbackify(async function(input, ctx, options) {
+jsonld.flatten = async function(input, ctx, options) {
   if(arguments.length < 1) {
     return new TypeError('Could not flatten, too few arguments.');
   }
@@ -375,7 +377,9 @@ jsonld.flatten = util.callbackify(async function(input, ctx, options) {
 
   // set default options
   options = _setDefaults(options, {
-    base: _isString(input) ? input : ''
+    base: _isString(input) ? input : '',
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
   });
 
   // expand input
@@ -395,7 +399,7 @@ jsonld.flatten = util.callbackify(async function(input, ctx, options) {
   const compacted = await jsonld.compact(flattened, ctx, options);
 
   return compacted;
-});
+};
 
 /**
  * Performs JSON-LD framing.
@@ -410,12 +414,12 @@ jsonld.flatten = util.callbackify(async function(input, ctx, options) {
  *          [explicit] default @explicit flag (default: false).
  *          [requireAll] default @requireAll flag (default: true).
  *          [omitDefault] default @omitDefault flag (default: false).
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
- * @param [callback(err, framed)] called once the operation completes.
+ *          [documentLoader(url, options)] the document loader.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the framed output.
  */
-jsonld.frame = util.callbackify(async function(input, frame, options) {
+jsonld.frame = async function(input, frame, options) {
   if(arguments.length < 2) {
     throw new TypeError('Could not frame, too few arguments.');
   }
@@ -423,12 +427,13 @@ jsonld.frame = util.callbackify(async function(input, frame, options) {
   // set default options
   options = _setDefaults(options, {
     base: _isString(input) ? input : '',
-    embed: '@last',
+    embed: '@once',
     explicit: false,
-    requireAll: true,
+    requireAll: false,
     omitDefault: false,
-    pruneBlankNodeIdentifiers: true,
-    bnodesToClear: []
+    bnodesToClear: [],
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
   });
 
   // if frame is a string, attempt to dereference remote document
@@ -453,33 +458,49 @@ jsonld.frame = util.callbackify(async function(input, frame, options) {
 
   const frameContext = frame ? frame['@context'] || {} : {};
 
+  // process context
+  const activeCtx = await jsonld.processContext(
+    _getInitialContext(options), frameContext, options);
+
+  // mode specific defaults
+  if(!options.hasOwnProperty('omitGraph')) {
+    options.omitGraph = _processingMode(activeCtx, 1.1);
+  }
+  if(!options.hasOwnProperty('pruneBlankNodeIdentifiers')) {
+    options.pruneBlankNodeIdentifiers = _processingMode(activeCtx, 1.1);
+  }
+
   // expand input
   const expanded = await jsonld.expand(input, options);
 
   // expand frame
-  const opts = util.clone(options);
+  const opts = {...options};
   opts.isFrame = true;
   opts.keepFreeFloatingNodes = true;
   const expandedFrame = await jsonld.expand(frame, opts);
 
   // if the unexpanded frame includes a key expanding to @graph, frame the
   // default graph, otherwise, the merged graph
-  let framed;
-  // FIXME should look for aliases of @graph
-  opts.merged = !('@graph' in frame);
-  // do framing
-  framed = _frameMergedOrDefault(expanded, expandedFrame, opts);
+  const frameKeys = Object.keys(frame)
+    .map(key => _expandIri(activeCtx, key, {vocab: true}));
+  opts.merged = !frameKeys.includes('@graph');
+  opts.is11 = _processingMode(activeCtx, 1.1);
 
-  // compact result (force @graph option to true, skip expansion,
-  // check for linked embeds)
-  opts.graph = true;
+  // do framing
+  const framed = _frameMergedOrDefault(expanded, expandedFrame, opts);
+
+  opts.graph = !options.omitGraph;
   opts.skipExpansion = true;
   opts.link = {};
   opts.framing = true;
-  const compacted = await jsonld.compact(framed, frameContext, opts);
+  let compacted = await jsonld.compact(framed, frameContext, opts);
+
+  // replace @null with null, compacting arrays
+  opts.link = {};
+  compacted = _cleanupNull(compacted, opts);
 
   return compacted;
-});
+};
 
 /**
  * **Experimental**
@@ -491,12 +512,12 @@ jsonld.frame = util.callbackify(async function(input, frame, options) {
  * @param [options] the options to use:
  *          [base] the base IRI to use.
  *          [expandContext] a context to expand with.
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
- * @param [callback(err, linked)] called once the operation completes.
+ *          [documentLoader(url, options)] the document loader.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the linked output.
  */
-jsonld.link = util.callbackify(async function(input, ctx, options) {
+jsonld.link = async function(input, ctx, options) {
   // API matches running frame with a wildcard frame and embed: '@link'
   // get arguments
   const frame = {};
@@ -505,7 +526,7 @@ jsonld.link = util.callbackify(async function(input, ctx, options) {
   }
   frame['@embed'] = '@link';
   return jsonld.frame(input, frame, options);
-});
+};
 
 /**
  * Performs RDF dataset normalization on the given input. The input is JSON-LD
@@ -525,14 +546,13 @@ jsonld.link = util.callbackify(async function(input, ctx, options) {
  *            'application/n-quads' for N-Quads.
  *          [format] the format if output is a string:
  *            'application/n-quads' for N-Quads.
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
+ *          [documentLoader(url, options)] the document loader.
  *          [useNative] true to use a native canonize algorithm
- * @param [callback(err, normalized)] called once the operation completes.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the normalized output.
  */
-jsonld.normalize = jsonld.canonize = util.callbackify(async function(
-  input, options) {
+jsonld.normalize = jsonld.canonize = async function(input, options) {
   if(arguments.length < 1) {
     throw new TypeError('Could not canonize, too few arguments.');
   }
@@ -541,7 +561,9 @@ jsonld.normalize = jsonld.canonize = util.callbackify(async function(
   options = _setDefaults(options, {
     base: _isString(input) ? input : '',
     algorithm: 'URDNA2015',
-    skipExpansion: false
+    skipExpansion: false,
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
   });
   if('inputFormat' in options) {
     if(options.inputFormat !== 'application/n-quads' &&
@@ -558,14 +580,14 @@ jsonld.normalize = jsonld.canonize = util.callbackify(async function(
   }
 
   // convert to RDF dataset then do normalization
-  const opts = util.clone(options);
+  const opts = {...options};
   delete opts.format;
   opts.produceGeneralizedRdf = false;
   const dataset = await jsonld.toRDF(input, opts);
 
   // do canonicalization
   return canonize.canonize(dataset, options);
-});
+};
 
 /**
  * Converts an RDF dataset to JSON-LD.
@@ -580,11 +602,10 @@ jsonld.normalize = jsonld.canonize = util.callbackify(async function(
  *            (default: false).
  *          [useNativeTypes] true to convert XSD types into native types
  *            (boolean, integer, double), false not to (default: false).
- * @param [callback(err, output)] called once the operation completes.
  *
  * @return a Promise that resolves to the JSON-LD document.
  */
-jsonld.fromRDF = util.callbackify(async function(dataset, options) {
+jsonld.fromRDF = async function(dataset, options) {
   if(arguments.length < 1) {
     throw new TypeError('Could not convert from RDF, too few arguments.');
   }
@@ -594,7 +615,8 @@ jsonld.fromRDF = util.callbackify(async function(dataset, options) {
     format: _isString(dataset) ? 'application/n-quads' : undefined
   });
 
-  let {format, rdfParser} = options;
+  const {format} = options;
+  let {rdfParser} = options;
 
   // handle special format
   if(format) {
@@ -610,34 +632,10 @@ jsonld.fromRDF = util.callbackify(async function(dataset, options) {
     rdfParser = () => dataset;
   }
 
-  // TODO: call `normalizeAsyncFn` on parser fn
-
-  // rdfParser can be callback, promise-based, or synchronous
-  let parsedDataset;
-  if(rdfParser.length > 1) {
-    // convert callback-based rdf parser to promise-based
-    parsedDataset = new Promise((resolve, reject) => {
-      rdfParser(dataset, (err, dataset) => {
-        if(err) {
-          reject(err);
-        } else {
-          resolve(dataset);
-        }
-      });
-    });
-  } else {
-    parsedDataset = Promise.resolve(rdfParser(dataset));
-  }
-
-  parsedDataset = await parsedDataset;
-
-  // back-compat with old parsers that produced legacy dataset format
-  if(!Array.isArray(parsedDataset)) {
-    parsedDataset = NQuads.legacyDatasetToQuads(parsedDataset);
-  }
-
+  // rdfParser must be synchronous or return a promise, no callback support
+  const parsedDataset = await rdfParser(dataset);
   return _fromRDF(parsedDataset, options);
-});
+};
 
 /**
  * Outputs the RDF dataset found in the given JSON-LD object.
@@ -652,12 +650,12 @@ jsonld.fromRDF = util.callbackify(async function(dataset, options) {
  *            'application/n-quads' for N-Quads.
  *          [produceGeneralizedRdf] true to output generalized RDF, false
  *            to produce only standard RDF (default: false).
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
- * @param [callback(err, dataset)] called once the operation completes.
+ *          [documentLoader(url, options)] the document loader.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the RDF dataset.
  */
-jsonld.toRDF = util.callbackify(async function(input, options) {
+jsonld.toRDF = async function(input, options) {
   if(arguments.length < 1) {
     throw new TypeError('Could not convert to RDF, too few arguments.');
   }
@@ -665,7 +663,9 @@ jsonld.toRDF = util.callbackify(async function(input, options) {
   // set default options
   options = _setDefaults(options, {
     base: _isString(input) ? input : '',
-    skipExpansion: false
+    skipExpansion: false,
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
   });
 
   // TODO: support toRDF custom map?
@@ -690,7 +690,7 @@ jsonld.toRDF = util.callbackify(async function(input, options) {
   }
 
   return dataset;
-});
+};
 
 /**
  * **Experimental**
@@ -703,26 +703,28 @@ jsonld.toRDF = util.callbackify(async function(input, options) {
  *          [base] the base IRI to use.
  *          [expandContext] a context to expand with.
  *          [issuer] a jsonld.IdentifierIssuer to use to label blank nodes.
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
- * @param [callback(err, nodeMap)] called once the operation completes.
+ *          [documentLoader(url, options)] the document loader.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the merged node map.
  */
-jsonld.createNodeMap = util.callbackify(async function(input, options) {
+jsonld.createNodeMap = async function(input, options) {
   if(arguments.length < 1) {
     throw new TypeError('Could not create node map, too few arguments.');
   }
 
   // set default options
   options = _setDefaults(options, {
-    base: _isString(input) ? input : ''
+    base: _isString(input) ? input : '',
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
   });
 
   // expand input
   const expanded = await jsonld.expand(input, options);
 
   return _createMergedNodeMap(expanded, options);
-});
+};
 
 /**
  * **Experimental**
@@ -740,12 +742,12 @@ jsonld.createNodeMap = util.callbackify(async function(input, options) {
  *            the ID has been defined; note that this may not prevent merging
  *            new properties where a node is in the `object` position
  *            (default: true).
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
- * @param [callback(err, merged)] called once the operation completes.
+ *          [documentLoader(url, options)] the document loader.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the merged output.
  */
-jsonld.merge = util.callbackify(async function(docs, ctx, options) {
+jsonld.merge = async function(docs, ctx, options) {
   if(arguments.length < 1) {
     throw new TypeError('Could not merge, too few arguments.');
   }
@@ -760,11 +762,14 @@ jsonld.merge = util.callbackify(async function(docs, ctx, options) {
   }
 
   // set default options
-  options = _setDefaults(options, {});
+  options = _setDefaults(options, {
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
+  });
 
   // expand all documents
   const expanded = await Promise.all(docs.map(doc => {
-    const opts = Object.assign({}, options);
+    const opts = {...options};
     return jsonld.expand(doc, opts);
   }));
 
@@ -829,22 +834,18 @@ jsonld.merge = util.callbackify(async function(docs, ctx, options) {
   const compacted = await jsonld.compact(flattened, ctx, options);
 
   return compacted;
-});
+};
 
 /**
- * The default document loader for external documents. If the environment
- * is node.js, a callback-continuation-style document loader is used; otherwise,
- * a promises-style document loader is used.
+ * The default document loader for external documents.
  *
  * @param url the URL to load.
- * @param callback(err, remoteDoc) called once the operation completes,
- *          if using a non-promises API.
  *
- * @return a promise, if using a promises API.
+ * @return a promise that resolves to the remote document.
  */
 Object.defineProperty(jsonld, 'documentLoader', {
   get: () => jsonld._documentLoader,
-  set: v => jsonld._documentLoader = util.normalizeDocumentLoader(v)
+  set: v => jsonld._documentLoader = v
 });
 // default document loader not implemented
 jsonld.documentLoader = async url => {
@@ -854,13 +855,6 @@ jsonld.documentLoader = async url => {
     {code: 'loading document failed', url});
 };
 
-/**
- * Deprecated default document loader. Do not use or override.
- */
-jsonld.loadDocument = util.callbackify(async function() {
-  return jsonld.documentLoader.apply(null, arguments);
-});
-
 /**
  * Gets a remote JSON-LD document using the default document loader or
  * one given in the passed options.
@@ -868,21 +862,19 @@ jsonld.loadDocument = util.callbackify(async function() {
  * @param url the URL to fetch.
  * @param [options] the options to use:
  *          [documentLoader] the document loader to use.
- * @param [callback(err, remoteDoc)] called once the operation completes.
  *
  * @return a Promise that resolves to the retrieved remote document.
  */
-jsonld.get = util.callbackify(async function(url, options) {
+jsonld.get = async function(url, options) {
   let load;
   if(typeof options.documentLoader === 'function') {
-    load = util.normalizeDocumentLoader(options.documentLoader);
+    load = options.documentLoader;
   } else {
     load = jsonld.documentLoader;
   }
 
   const remoteDoc = await load(url);
 
-  // TODO: can this be moved into `normalizeDocumentLoader`?
   try {
     if(!remoteDoc.document) {
       throw new JsonLdError(
@@ -903,25 +895,27 @@ jsonld.get = util.callbackify(async function(url, options) {
   }
 
   return remoteDoc;
-});
+};
 
 /**
  * Processes a local context, resolving any URLs as necessary, and returns a
- * new active context in its callback.
+ * new active context.
  *
  * @param activeCtx the current active context.
  * @param localCtx the local context to process.
  * @param [options] the options to use:
- *          [documentLoader(url, callback(err, remoteDoc))] the document loader.
- * @param [callback(err, activeCtx)] called once the operation completes.
+ *          [documentLoader(url, options)] the document loader.
+ *          [contextResolver] internal use only.
  *
  * @return a Promise that resolves to the new active context.
  */
-jsonld.processContext = util.callbackify(async function(
+jsonld.processContext = async function(
   activeCtx, localCtx, options) {
   // set default options
   options = _setDefaults(options, {
-    base: ''
+    base: '',
+    contextResolver: new ContextResolver(
+      {sharedCache: _resolvedContextCache})
   });
 
   // return initial context early for null context
@@ -934,10 +928,9 @@ jsonld.processContext = util.callbackify(async function(
   if(!(_isObject(localCtx) && '@context' in localCtx)) {
     localCtx = {'@context': localCtx};
   }
-  const ctx = await _getAllContexts(localCtx, options);
 
-  return _processContext({activeCtx, localCtx: ctx, options});
-});
+  return _processContext({activeCtx, localCtx, options});
+};
 
 // backwards compatibility
 jsonld.getContextValue = require('./context').getContextValue;
@@ -969,29 +962,15 @@ jsonld.useDocumentLoader = function(type) {
     jsonld, Array.prototype.slice.call(arguments, 1));
 };
 
-/** Registered RDF dataset parsers hashed by content-type. */
-const _rdfParsers = {};
-
 /**
  * Registers an RDF dataset parser by content-type, for use with
- * jsonld.fromRDF. An RDF dataset parser will always be given two parameters,
- * a string of input and a callback. An RDF dataset parser can be synchronous
- * or asynchronous.
- *
- * If the parser function returns undefined or null then it will be assumed to
- * be asynchronous w/a continuation-passing style and the callback parameter
- * given to the parser MUST be invoked.
- *
- * If it returns a Promise, then it will be assumed to be asynchronous, but the
- * callback parameter MUST NOT be invoked. It should instead be ignored.
- *
- * If it returns an RDF dataset, it will be assumed to be synchronous and the
- * callback parameter MUST NOT be invoked. It should instead be ignored.
+ * jsonld.fromRDF. An RDF dataset parser will always be given one parameter,
+ * a string of input. An RDF dataset parser can be synchronous or
+ * asynchronous (by returning a promise).
  *
  * @param contentType the content-type for the parser.
- * @param parser(input, callback(err, dataset)) the parser function (takes a
- *          string as a parameter and either returns null/undefined and uses
- *          the given callback, returns a Promise, or returns an RDF dataset).
+ * @param parser(input) the parser function (takes a string as a parameter
+ *          and either returns an RDF dataset or a Promise that resolves to one.
  */
 jsonld.registerRDFParser = function(contentType, parser) {
   _rdfParsers[contentType] = parser;
