@@ -1,16 +1,12 @@
 /**
- * Common test runner for JSON-LD.
- *
- * @author Dave Longley
- * @author David I. Lehn
- *
- * Copyright (c) 2011-2017 Digital Bazaar, Inc. All rights reserved.
+ * Copyright (c) 2011-2019 Digital Bazaar, Inc. All rights reserved.
  */
 /* eslint-disable indent */
 const EarlReport = require('./earl-report');
 const benchmark = require('benchmark');
 const join = require('join-path-js');
 const rdfCanonize = require('rdf-canonize');
+const {prependBase} = require('../lib/url');
 
 module.exports = function(options) {
 
@@ -36,48 +32,14 @@ const TEST_TYPES = {
       // see JSON-LD 1.0 Errata
       specVersion: ['json-ld-1.0'],
       // FIXME
+      // NOTE: idRegex format:
+      //MMM-manifest#tNNN$/,
       idRegex: [
-        // list of lists
-        /compact-manifest.jsonld#tli01$/,
-        /compact-manifest.jsonld#tli02$/,
-        /compact-manifest.jsonld#tli03$/,
-        /compact-manifest.jsonld#tli04$/,
-        /compact-manifest.jsonld#tli05$/,
-        // terms
-        /compact-manifest.jsonld#tp001$/,
-        // rel iri
-        /compact-manifest.jsonld#t0095$/,
-        // type set
-        /compact-manifest.jsonld#t0104$/,
-        /compact-manifest.jsonld#t0105$/,
-        // rel vocab
-        /compact-manifest.jsonld#t0107$/,
         // html
-        /compact-manifest.jsonld#th001$/,
-        /compact-manifest.jsonld#th002$/,
-        /compact-manifest.jsonld#th003$/,
-        /compact-manifest.jsonld#th004$/,
-        // @type: @none
-        /compact-manifest.jsonld#ttn01$/,
-        /compact-manifest.jsonld#ttn02$/,
-        /compact-manifest.jsonld#ttn03$/,
-        // property-valued indexes
-        /compact-manifest.jsonld#tpi01$/,
-        /compact-manifest.jsonld#tpi02$/,
-        /compact-manifest.jsonld#tpi03$/,
-        /compact-manifest.jsonld#tpi04$/,
-        /compact-manifest.jsonld#tpi05$/,
-        /compact-manifest.jsonld#tpi06$/,
-        // JSON literals
-        /compact-manifest.jsonld#tjs01$/,
-        /compact-manifest.jsonld#tjs02$/,
-        /compact-manifest.jsonld#tjs03$/,
-        /compact-manifest.jsonld#tjs04$/,
-        /compact-manifest.jsonld#tjs05$/,
-        /compact-manifest.jsonld#tjs06$/,
-        /compact-manifest.jsonld#tjs07$/,
-        /compact-manifest.jsonld#tjs08$/,
-        /compact-manifest.jsonld#tjs09$/,
+        /html-manifest#tc001$/,
+        /html-manifest#tc002$/,
+        /html-manifest#tc003$/,
+        /html-manifest#tc004$/,
       ]
     },
     fn: 'compact',
@@ -90,94 +52,43 @@ const TEST_TYPES = {
   },
   'jld:ExpandTest': {
     skip: {
+      // skip tests where behavior changed for a 1.1 processor
+      // see JSON-LD 1.0 Errata
+      specVersion: ['json-ld-1.0'],
       // FIXME
+      // NOTE: idRegex format:
+      //MMM-manifest#tNNN$/,
       idRegex: [
-        // list of lists
-        /expand-manifest.jsonld#tli01$/,
-        /expand-manifest.jsonld#tli02$/,
-        /expand-manifest.jsonld#tli03$/,
-        /expand-manifest.jsonld#tli04$/,
-        /expand-manifest.jsonld#tli05$/,
-        /expand-manifest.jsonld#tli06$/,
-        /expand-manifest.jsonld#tli07$/,
-        /expand-manifest.jsonld#tli08$/,
-        /expand-manifest.jsonld#tli09$/,
-        /expand-manifest.jsonld#tli10$/,
-        // mode
-        /expand-manifest.jsonld#tp001$/,
-        /expand-manifest.jsonld#tp002$/,
-        // rel iri
-        /expand-manifest.jsonld#t0092$/,
-        // iris
-        /expand-manifest.jsonld#t0109$/,
-        // rel vocab
-        /expand-manifest.jsonld#t0110$/,
-        /expand-manifest.jsonld#t0111$/,
-        /expand-manifest.jsonld#t0112$/,
         // html
-        /expand-manifest.jsonld#th001$/,
-        /expand-manifest.jsonld#th002$/,
-        /expand-manifest.jsonld#th003$/,
-        /expand-manifest.jsonld#th004$/,
-        /expand-manifest.jsonld#th005$/,
-        /expand-manifest.jsonld#th006$/,
-        /expand-manifest.jsonld#th007$/,
-        /expand-manifest.jsonld#th010$/,
-        /expand-manifest.jsonld#th011$/,
-        /expand-manifest.jsonld#th012$/,
-        /expand-manifest.jsonld#th013$/,
-        /expand-manifest.jsonld#th014$/,
-        /expand-manifest.jsonld#th015$/,
-        /expand-manifest.jsonld#th016$/,
-        /expand-manifest.jsonld#th017$/,
-        /expand-manifest.jsonld#th018$/,
-        /expand-manifest.jsonld#th019$/,
-        /expand-manifest.jsonld#th020$/,
-        /expand-manifest.jsonld#th021$/,
-        /expand-manifest.jsonld#th022$/,
+        /html-manifest#te001$/,
+        /html-manifest#te002$/,
+        /html-manifest#te003$/,
+        /html-manifest#te004$/,
+        /html-manifest#te005$/,
+        /html-manifest#te006$/,
+        /html-manifest#te007$/,
+        /html-manifest#te010$/,
+        /html-manifest#te011$/,
+        /html-manifest#te012$/,
+        /html-manifest#te013$/,
+        /html-manifest#te014$/,
+        /html-manifest#te015$/,
+        /html-manifest#te016$/,
+        /html-manifest#te017$/,
+        /html-manifest#te018$/,
+        /html-manifest#te019$/,
+        /html-manifest#te020$/,
+        /html-manifest#te021$/,
+        /html-manifest#te022$/,
+        /html-manifest#tex01$/,
         // HTML extraction
-        /expand-manifest.jsonld#thc01$/,
-        /expand-manifest.jsonld#thc02$/,
-        /expand-manifest.jsonld#thc03$/,
-        /expand-manifest.jsonld#thc04$/,
-        /expand-manifest.jsonld#thc05$/,
-        // @type: @none
-        /expand-manifest.jsonld#ttn02$/,
-        // property index maps
-        /expand-manifest.jsonld#tpi01$/,
-        /expand-manifest.jsonld#tpi02$/,
-        /expand-manifest.jsonld#tpi03$/,
-        /expand-manifest.jsonld#tpi04$/,
-        /expand-manifest.jsonld#tpi05$/,
-        /expand-manifest.jsonld#tpi06$/,
-        /expand-manifest.jsonld#tpi07$/,
-        /expand-manifest.jsonld#tpi08$/,
-        /expand-manifest.jsonld#tpi09$/,
-        /expand-manifest.jsonld#tpi10$/,
-        /expand-manifest.jsonld#tpi11$/,
-        // JSON literals
-        /expand-manifest.jsonld#tjs01$/,
-        /expand-manifest.jsonld#tjs02$/,
-        /expand-manifest.jsonld#tjs03$/,
-        /expand-manifest.jsonld#tjs04$/,
-        /expand-manifest.jsonld#tjs05$/,
-        /expand-manifest.jsonld#tjs06$/,
-        /expand-manifest.jsonld#tjs07$/,
-        /expand-manifest.jsonld#tjs08$/,
-        /expand-manifest.jsonld#tjs09$/,
-        /expand-manifest.jsonld#tjs10$/,
-        /expand-manifest.jsonld#tjs11$/,
-        // misc
-        /expand-manifest.jsonld#te043$/,
-        /expand-manifest.jsonld#te044$/,
+        /expand-manifest#thc01$/,
+        /expand-manifest#thc02$/,
+        /expand-manifest#thc03$/,
+        /expand-manifest#thc04$/,
+        /expand-manifest#thc05$/,
         // remote
-        /remote-doc-manifest.jsonld#t0005$/,
-        /remote-doc-manifest.jsonld#t0006$/,
-        /remote-doc-manifest.jsonld#t0007$/,
-        /remote-doc-manifest.jsonld#t0010$/,
-        /remote-doc-manifest.jsonld#t0011$/,
-        /remote-doc-manifest.jsonld#t0012$/,
-        /remote-doc-manifest.jsonld#t0013$/,
+        /remote-doc-manifest#t0013$/, // HTML
       ]
     },
     fn: 'expand',
@@ -189,17 +100,18 @@ const TEST_TYPES = {
   },
   'jld:FlattenTest': {
     skip: {
+      // skip tests where behavior changed for a 1.1 processor
+      // see JSON-LD 1.0 Errata
+      specVersion: ['json-ld-1.0'],
       // FIXME
+      // NOTE: idRegex format:
+      //MMM-manifest#tNNN$/,
       idRegex: [
-        // list of lists
-        /flatten-manifest.jsonld#tli01$/,
-        /flatten-manifest.jsonld#tli02$/,
-        /flatten-manifest.jsonld#tli03$/,
         // html
-        /flatten-manifest.jsonld#th001$/,
-        /flatten-manifest.jsonld#th002$/,
-        /flatten-manifest.jsonld#th003$/,
-        /flatten-manifest.jsonld#th004$/,
+        /html-manifest#tf001$/,
+        /html-manifest#tf002$/,
+        /html-manifest#tf003$/,
+        /html-manifest#tf004$/,
       ]
     },
     fn: 'flatten',
@@ -212,37 +124,13 @@ const TEST_TYPES = {
   },
   'jld:FrameTest': {
     skip: {
+      // skip tests where behavior changed for a 1.1 processor
+      // see JSON-LD 1.0 Errata
+      specVersion: ['json-ld-1.0'],
       // FIXME
+      // NOTE: idRegex format:
+      //MMM-manifest#tNNN$/,
       idRegex: [
-        // ex
-        /frame-manifest.jsonld#tg001$/,
-        // graphs
-        /frame-manifest.jsonld#t0010$/,
-        /frame-manifest.jsonld#t0020$/,
-        /frame-manifest.jsonld#t0046$/,
-        /frame-manifest.jsonld#t0049$/,
-        /frame-manifest.jsonld#t0051$/,
-        /frame-manifest.jsonld#tg010$/,
-        /frame-manifest.jsonld#tp046$/,
-        /frame-manifest.jsonld#tp049$/,
-        // blank nodes
-        /frame-manifest.jsonld#t0052$/,
-        /frame-manifest.jsonld#t0053$/,
-        // embed
-        /frame-manifest.jsonld#t0054$/,
-        // lists
-        /frame-manifest.jsonld#t0055$/,
-        /frame-manifest.jsonld#t0058$/,
-        // misc
-        /frame-manifest.jsonld#tp010$/,
-        /frame-manifest.jsonld#tp050$/,
-        /frame-manifest.jsonld#teo01$/,
-        // @embed:@first
-        /frame-manifest.jsonld#t0060$/,
-        // requireAll
-        /frame-manifest.jsonld#tra01$/,
-        /frame-manifest.jsonld#tra02$/,
-        /frame-manifest.jsonld#tra03$/,
       ]
     },
     fn: 'frame',
@@ -255,20 +143,16 @@ const TEST_TYPES = {
   },
   'jld:FromRDFTest': {
     skip: {
+      // skip tests where behavior changed for a 1.1 processor
+      // see JSON-LD 1.0 Errata
+      specVersion: ['json-ld-1.0'],
       // FIXME
+      // NOTE: idRegex format:
+      //MMM-manifest#tNNN$/,
       idRegex: [
-        // list of lists
-        /fromRdf-manifest.jsonld#tli01$/,
-        /fromRdf-manifest.jsonld#tli02$/,
-        /fromRdf-manifest.jsonld#tli03$/,
-        // JSON literals
-        /fromRdf-manifest.jsonld#tjs01$/,
-        /fromRdf-manifest.jsonld#tjs02$/,
-        /fromRdf-manifest.jsonld#tjs03$/,
-        /fromRdf-manifest.jsonld#tjs04$/,
-        /fromRdf-manifest.jsonld#tjs05$/,
-        /fromRdf-manifest.jsonld#tjs06$/,
-        /fromRdf-manifest.jsonld#tjs07$/,
+        // direction (compound-literal)
+        /fromRdf-manifest#tdi11$/,
+        /fromRdf-manifest#tdi12$/,
       ]
     },
     fn: 'fromRDF',
@@ -288,41 +172,44 @@ const TEST_TYPES = {
   },
   'jld:ToRDFTest': {
     skip: {
+      // skip tests where behavior changed for a 1.1 processor
+      // see JSON-LD 1.0 Errata
+      specVersion: ['json-ld-1.0'],
       // FIXME
+      // NOTE: idRegex format:
+      //MMM-manifest#tNNN$/,
       idRegex: [
-        // list of lists
-        /toRdf-manifest.jsonld#tli01$/,
-        /toRdf-manifest.jsonld#tli02$/,
-        // blank node properties
-        /toRdf-manifest.jsonld#t0118$/,
         // well formed
-        /toRdf-manifest.jsonld#twf01$/,
-        /toRdf-manifest.jsonld#twf02$/,
-        /toRdf-manifest.jsonld#twf03$/,
-        /toRdf-manifest.jsonld#twf04$/,
-        /toRdf-manifest.jsonld#twf05$/,
-        /toRdf-manifest.jsonld#twf06$/,
-        /toRdf-manifest.jsonld#twf07$/,
+        /toRdf-manifest#twf05$/,
+
         // html
-        /toRdf-manifest.jsonld#th001$/,
-        /toRdf-manifest.jsonld#th002$/,
-        /toRdf-manifest.jsonld#th003$/,
-        // JSON literal
-        /toRdf-manifest.jsonld#tjs01$/,
-        /toRdf-manifest.jsonld#tjs02$/,
-        /toRdf-manifest.jsonld#tjs03$/,
-        /toRdf-manifest.jsonld#tjs04$/,
-        /toRdf-manifest.jsonld#tjs05$/,
-        /toRdf-manifest.jsonld#tjs06$/,
-        /toRdf-manifest.jsonld#tjs07$/,
-        /toRdf-manifest.jsonld#tjs08$/,
-        /toRdf-manifest.jsonld#tjs09$/,
-        /toRdf-manifest.jsonld#tjs10$/,
-        /toRdf-manifest.jsonld#tjs11$/,
-        /toRdf-manifest.jsonld#tjs12$/,
-        /toRdf-manifest.jsonld#tjs13$/,
-        // number fixes
-        /toRdf-manifest.jsonld#trt01$/,
+        /html-manifest#tr001$/,
+        /html-manifest#tr002$/,
+        /html-manifest#tr003$/,
+        /html-manifest#tr004$/,
+        /html-manifest#tr005$/,
+        /html-manifest#tr006$/,
+        /html-manifest#tr007$/,
+        /html-manifest#tr010$/,
+        /html-manifest#tr011$/,
+        /html-manifest#tr012$/,
+        /html-manifest#tr013$/,
+        /html-manifest#tr014$/,
+        /html-manifest#tr015$/,
+        /html-manifest#tr016$/,
+        /html-manifest#tr017$/,
+        /html-manifest#tr018$/,
+        /html-manifest#tr019$/,
+        /html-manifest#tr020$/,
+        /html-manifest#tr021$/,
+        /html-manifest#tr022$/,
+        // Invalid Statement
+        /toRdf-manifest#te075$/,
+        /toRdf-manifest#te111$/,
+        /toRdf-manifest#te112$/,
+        // direction (compound-literal)
+        /toRdf-manifest#tdi11$/,
+        /toRdf-manifest#tdi12$/,
       ]
     },
     fn: 'toRDF',
@@ -501,7 +388,10 @@ function addTest(manifest, test, tests) {
   // expand @id and input base
   const test_id = test['@id'] || test['id'];
   //var number = test_id.substr(2);
-  test['@id'] = manifest.baseIri + basename(manifest.filename) + test_id;
+  test['@id'] =
+    manifest.baseIri +
+    basename(manifest.filename).replace('.jsonld', '') +
+    test_id;
   test.base = manifest.baseIri + test.input;
   test.manifest = manifest;
   const description = test_id + ' ' + (test.purpose || test.name);
@@ -594,6 +484,12 @@ function addTest(manifest, test, tests) {
         });
       }
 
+      // Make expandContext absolute to the manifest
+      if(test.hasOwnProperty('option') && test.option.expandContext) {
+        test.option.expandContext =
+          prependBase(test.manifest.baseIri, test.option.expandContext);
+      }
+
       const testOptions = getJsonLdValues(test, 'option');
 
       testOptions.forEach(function(opt) {
@@ -687,7 +583,7 @@ function addTest(manifest, test, tests) {
               .on('error', err => {
                 reject(new Error(err));
               })
-              .on('complete', e => {
+              .on('complete', () => {
                 resolve();
               })
               .run({async: true});
@@ -825,7 +721,9 @@ function createTestOptions(opts) {
 
 // find the expected output property or throw error
 function _getExpectProperty(test) {
-  if('expect' in test) {
+  if('expectErrorCode' in test) {
+    return 'expectErrorCode';
+  } else if('expect' in test) {
     return 'expect';
   } else if('result' in test) {
     return 'result';
@@ -890,15 +788,17 @@ async function compareExpectedError(test, err) {
   try {
     expect = test[_getExpectProperty(test)];
     result = getJsonLdErrorCode(err);
-    assert.ok(err);
+    assert.ok(err, 'no error present');
     assert.strictEqual(result, expect);
-  } catch(err) {
+  } catch(_err) {
     if(options.bailOnError) {
       console.log('\nTEST FAILED\n');
       console.log('EXPECTED: ' + expect);
       console.log('ACTUAL: ' + result);
     }
-    throw err;
+    // log the unexpected error to help with debugging
+    console.log('Unexpected error:', err);
+    throw _err;
   }
 }
 
@@ -991,11 +891,12 @@ function createDocumentLoader(test) {
     'https://w3c.github.io/json-ld-api/tests',
     'https://w3c.github.io/json-ld-framing/tests'
   ];
-  const localLoader = function(url, callback) {
+  const localLoader = function(url) {
     // always load remote-doc tests remotely in node
-    if(options.nodejs && test.manifest.name === 'Remote document') {
-      return jsonld.loadDocument(url, callback);
-    }
+    // NOTE: disabled due to github pages issues.
+    //if(options.nodejs && test.manifest.name === 'Remote document') {
+    //  return jsonld.documentLoader(url);
+    //}
 
     // FIXME: this check only works for main test suite and will not work if:
     // - running other tests and main test suite not installed
@@ -1007,13 +908,11 @@ function createDocumentLoader(test) {
     const isRelative = url.indexOf(':') === -1;
     if(isTestSuite || isRelative) {
       // attempt to load official test-suite files or relative URLs locally
-      loadLocally(url).then(callback.bind(null, null), callback);
-      // don't return the promise
-      return;
+      return loadLocally(url);
     }
 
     // load remotely
-    return jsonld.loadDocument(url, callback);
+    return jsonld.documentLoader(url);
   };
 
   return localLoader;
@@ -1029,17 +928,28 @@ function createDocumentLoader(test) {
         if(!contentType && url.indexOf('.jsonld', url.length - 7) !== -1) {
           contentType = 'application/ld+json';
         }
+        if(!contentType && url.indexOf('.json', url.length - 5) !== -1) {
+          contentType = 'application/json';
+        }
         let linkHeader = options.httpLink;
         if(Array.isArray(linkHeader)) {
           linkHeader = linkHeader.join(',');
         }
-        linkHeader = jsonld.parseLinkHeader(
-          linkHeader)['http://www.w3.org/ns/json-ld#context'];
-        if(linkHeader && contentType !== 'application/ld+json') {
-          if(Array.isArray(linkHeader)) {
+        const linkHeaders = jsonld.parseLinkHeader(linkHeader);
+        const linkedContext =
+          linkHeaders['http://www.w3.org/ns/json-ld#context'];
+        if(linkedContext && contentType !== 'application/ld+json') {
+          if(Array.isArray(linkedContext)) {
             throw {name: 'multiple context link headers'};
           }
-          doc.contextUrl = linkHeader.target;
+          doc.contextUrl = linkedContext.target;
+        }
+
+        // If not JSON-LD, alternate may point there
+        if(linkHeaders['alternate'] &&
+          linkHeaders['alternate'].type == 'application/ld+json' &&
+          !(contentType || '').match(/^application\/(\w*\+)?json$/)) {
+          doc.documentUrl = prependBase(url, linkHeaders['alternate'].target);
         }
       }
     }
@@ -1065,7 +975,7 @@ function createDocumentLoader(test) {
     return p.then(readJson).then(json => {
       doc.document = json;
       return doc;
-    }).catch(err => {
+    }).catch(() => {
       throw {name: 'loading document failed', url};
     });
   }
