@@ -22,6 +22,7 @@ const outputs = [
       'core-js/fn/array/includes',
       'core-js/fn/map',
       'core-js/fn/object/assign',
+      'core-js/fn/object/entries',
       'core-js/fn/promise',
       'core-js/fn/set',
       'core-js/fn/string/starts-with',
@@ -64,8 +65,11 @@ outputs.forEach(info => {
             // exclude node_modules by default
             exclude: /(node_modules)/
           }, {
-            // include rdf-canonize
-            include: /(node_modules\/rdf-canonize)/
+            // include specific packages
+            include: [
+              /(node_modules\/canonicalize)/,
+              /(node_modules\/rdf-canonize)/
+            ]
           }],
           use: {
             loader: 'babel-loader',
